# utils.py

import codecs, bz2, gzip

def open_with_unicode(file_name, compression_type, mode):
    assert compression_type in [None, 'gzip', 'bzip2']
    assert mode in ['r', 'w']
    if compression_type == None:
        if mode == 'r':
            return codecs.getreader('utf-8')(open(file_name, mode))
        elif mode == 'w':
            return codecs.getwriter('utf-8')(open(file_name, mode))
    elif compression_type == 'gzip':
        if mode == 'r':
            return codecs.getreader('utf-8')(gzip.GzipFile(file_name, mode))
        elif mode == 'w':
            return codecs.getwriter('utf-8')(gzip.GzipFile(file_name, mode))
    elif compression_type == 'bzip2':
        if mode == 'r':
            return codecs.getreader('utf-8')(bz2.BZ2File(file_name, mode))
        elif mode == 'w':
            return codecs.getwriter('utf-8')(bz2.BZ2File(file_name, mode))


def split_file_into_chunks(file_name, directory, lines_per_chunk):

    """
    Assumes bzip2 compression.
    """

    file_obj = open_with_unicode(file_name, 'bzip2', 'r')
    current_line_number = 0
    current_file_number = 0
    end_of_file = False
    while not end_of_file:
        current_filename = directory + '%03d' % current_file_number + '.bz2'
        current_file_obj = open_with_unicode(current_filename, 'bzip2', 'w')
        current_file_obj.write(file_obj.readline())
        current_line_number += 1
        while current_line_number % lines_per_chunk > 0:
            current_line = file_obj.readline()
            end_of_file = current_line == ''
            current_file_obj.write(current_line)
            current_line_number += 1
        current_file_number += 1
    return




# test_vocabulary_generator.py

from recluse import vocabulary_generator
from recluse.utils import split_file_into_chunks
import unittest, StringIO, subprocess, os, shutil

size = 20
file_names = ['recluse/test/data/f1.bz2', 'recluse/test/data/f2.bz2', 'recluse/test/data/f3.bz2']

class VocabularyGeneratorTest(unittest.TestCase):

    def test_vocabulary_generator(self):

        vg = vocabulary_generator.VocabularyGenerator(file_names)
        out_file_obj = StringIO.StringIO()
        vg.generate_vocabulary(size, out_file_obj)
        words = out_file_obj.getvalue()
        self.assertEqual(words, u',\n"\nthe\nof\n<s>\n</s>\n.\nand\n-\nanarchism\na\nto\nas\nin\nor\nis\nfrom\nsome\nanarchists\n)\n'), words

    def test_command_line(self):
        text_file_name = 'recluse/test/data/small_westbury.txt.bz2'
        vg = vocabulary_generator.VocabularyGenerator(text_file_name)
        temporary_chunk_directory = 'TEMP_CHUNK_DIR/'
        os.mkdir(temporary_chunk_directory)
        lines_per_chunk = 500
        split_file_into_chunks(text_file_name, temporary_chunk_directory, lines_per_chunk) 
        file_names = [s for s in os.listdir(temporary_chunk_directory)]

        generate_vocab = subprocess.Popen(['python', 'recluse/vocabulary_generator.py', str(size)] + [temporary_chunk_directory + '/' + fn for fn in file_names], stdin=-1, stdout=-1, stderr=-1)
        (stdoutdata, stderrdata) = generate_vocab.communicate()
        self.assertEqual(generate_vocab.returncode, 0)
        self.assertEqual(stdoutdata, 'the\n,\n<s>\n</s>\n.\nof\nand\nin\n"\nto\na\nis\n-\nas\nthat\n<4-digit-integer>\nfor\nwas\nwith\n)\n'), stdoutdata
        
        shutil.rmtree(temporary_chunk_directory)

if __name__ == '__main__':
    unittest.main()



# vocabulary_generator.py

import subprocess, os, stat, shutil, sys
from utils import *
import vocabulary_cutter

class VocabularyGenerator():

    """
    Uses the SRILM toolkit to generate unigram counts and then cuts
    down to size with the most frequent words.
    """

    def __init__(self, file_names):
        self.file_names= file_names

    def srilm_make_merge_batch_counts(self, directory):

        file_names_file_obj = open(directory + 'file_names', 'w')
        file_names_file_obj.writelines([s + '\n' for s in self.file_names])
        file_names_file_obj.close()

        cwd = os.getcwd()

        bash_script_name = cwd + "/nltkbasedsegmentandtokenise.sh"
        bash_script_file_obj = open(bash_script_name, 'w')
        bash_script_file_obj.write("bzcat $1 | nltkbasedsegmentertokeniserrunner\n")
        bash_script_file_obj.close()
        os.chmod(bash_script_name, 0755)

        srilm_make_batch_counts = subprocess.call(['make-batch-counts', directory + 'file_names', '1', bash_script_name, directory, '-write-order 1'])
        srilm_merge_batch_counts = subprocess.call(['merge-batch-counts', directory])

        os.remove(bash_script_name)

    def generate_vocabulary(self, size, vocabulary_file_obj):
        
        temporary_directory = 'TEMP_DIR/'
        if not os.path.isdir(temporary_directory):
            os.mkdir(temporary_directory)
        self.srilm_make_merge_batch_counts(temporary_directory)
        merged_counts_file = [f for f in os.listdir(temporary_directory) if f.endswith('.ngrams.gz')][0]
        unigram_counts_file_obj = open_with_unicode(temporary_directory + merged_counts_file, 'gzip', 'r')
        cutter = vocabulary_cutter.VocabularyCutter(unigram_counts_file_obj, vocabulary_file_obj)
        cutter.cut_vocabulary(size)
        
        shutil.rmtree(temporary_directory)


if __name__ == '__main__':
    size = int(sys.argv[1])
    file_names = sys.argv[2:]
    outfile_obj = sys.stdout
    vg = VocabularyGenerator(file_names)
    vg.generate_vocabulary(size, outfile_obj)



# 2012 L. Amber Wilcox-O'Hearn

import sys, bisect

class VocabularyCutter():
    """
    Takes a file of unigram counts and returns a file of the top n
    most frequent words.
    If there are fewer than n words, it returns them all.
    """
    def __init__(self, infile_obj, outfile_obj):
        self.infile_obj = infile_obj
        self.outfile_obj = outfile_obj

    def cut_vocabulary(self, n):
        frequencies = []
        for line in self.infile_obj:
            tokens = line.split()
            assert len(tokens) == 2, line
            bisect.insort( frequencies, (int(tokens[1]), tokens[0]) )
        n = min(n, len(frequencies))
        for i in range(n):
            self.outfile_obj.write(frequencies[-1-i][1])
            self.outfile_obj.write('\n')

if __name__ == '__main__':
    infile_obj = sys.stdin
    outfile_obj = sys.stdout
    vc = VocabularyCutter(infile_obj, outfile_obj)
    vc.cut_vocabulary(int(sys.argv[1]))
